@@ -63,6 +63,7 @@
 #include "_psutil_posix.h"
 
 #ifdef PSUTIL_FREEBSD
+    #include "arch/freebsd/cpu.h"
     #include "arch/freebsd/specific.h"
     #include "arch/freebsd/sys_socks.h"
     #include "arch/freebsd/proc_socks.h"
@@ -1096,8 +1097,8 @@ static PyMethodDef mod_methods[] = {
      "Get process resource limits."},
     {"proc_setrlimit", psutil_proc_setrlimit, METH_VARARGS,
      "Set process resource limits."},
-    {"cpu_count_phys", psutil_cpu_count_phys, METH_VARARGS,
-     "Return an XML string to determine the number physical CPUs."},
+    {"cpu_topology", psutil_cpu_topology, METH_VARARGS,
+     "Return CPU topology as an XML string."},
 #endif
     {"proc_environ", psutil_proc_environ, METH_VARARGS,
      "Return process environment"},
@@ -1143,8 +1144,8 @@ static PyMethodDef mod_methods[] = {
 #endif
 
     // --- others
-    {"set_testing", psutil_set_testing, METH_NOARGS,
-     "Set psutil in testing mode"},
+    {"set_debug", psutil_set_debug, METH_VARARGS,
+     "Enable or disable PSUTIL_DEBUG messages"},
 
     {NULL, NULL, 0, NULL}
 };
