@@ -43,35 +43,6 @@ static PyObject *TimeoutExpired;
 static PyObject *TimeoutAbandoned;
 
 
-/*
- * Return the number of logical, active CPUs. Return 0 if undetermined.
- * See discussion at: https://bugs.python.org/issue33166#msg314631
- */
-unsigned int
-psutil_get_num_cpus(int fail_on_err) {
-    unsigned int ncpus = 0;
-
-    // Minimum requirement: Windows 7
-    if (GetActiveProcessorCount != NULL) {
-        ncpus = GetActiveProcessorCount(ALL_PROCESSOR_GROUPS);
-        if ((ncpus == 0) && (fail_on_err == 1)) {
-            PyErr_SetFromWindowsErr(0);
-        }
-    }
-    else {
-        psutil_debug("GetActiveProcessorCount() not available; "
-                     "using GetSystemInfo()");
-        ncpus = (unsigned int)PSUTIL_SYSTEM_INFO.dwNumberOfProcessors;
-        if ((ncpus <= 0) && (fail_on_err == 1)) {
-            PyErr_SetString(
-                PyExc_RuntimeError,
-                "GetSystemInfo() failed to retrieve CPU count");
-        }
-    }
-    return ncpus;
-}
-
-
 /*
  * Return a Python float representing the system uptime expressed in seconds
  * since the epoch.
@@ -367,8 +338,8 @@ static PyObject *
 psutil_proc_exe(PyObject *self, PyObject *args) {
     DWORD pid;
     NTSTATUS status;
-    PVOID buffer;
-    ULONG bufferSize = 0x100;
+    PVOID buffer = NULL;
+    ULONG bufferSize = 0x104 * 2; // WIN_MAX_PATH * sizeof(wchar_t)
     SYSTEM_PROCESS_ID_INFORMATION processIdInfo;
     PyObject *py_exe;
 
@@ -379,8 +350,11 @@ psutil_proc_exe(PyObject *self, PyObject *args) {
         return AccessDenied("automatically set for PID 0");
 
     buffer = MALLOC_ZERO(bufferSize);
-    if (! buffer)
-        return PyErr_NoMemory();
+    if (! buffer) {
+        PyErr_NoMemory();
+        return NULL;
+    }
+
     processIdInfo.ProcessId = (HANDLE)(ULONG_PTR)pid;
     processIdInfo.ImageName.Length = 0;
     processIdInfo.ImageName.MaximumLength = (USHORT)bufferSize;
@@ -392,12 +366,41 @@ psutil_proc_exe(PyObject *self, PyObject *args) {
         sizeof(SYSTEM_PROCESS_ID_INFORMATION),
         NULL);
 
-    if (status == STATUS_INFO_LENGTH_MISMATCH) {
+    if ((status == STATUS_INFO_LENGTH_MISMATCH) &&
+            (processIdInfo.ImageName.MaximumLength <= bufferSize))
+    {
+        // Required length was NOT stored in MaximumLength (WOW64 issue).
+        ULONG maxBufferSize = 0x7FFF * 2;  // NTFS_MAX_PATH * sizeof(wchar_t)
+        do {
+            // Iteratively double the size of the buffer up to maxBufferSize
+            bufferSize *= 2;
+            FREE(buffer);
+            buffer = MALLOC_ZERO(bufferSize);
+            if (! buffer) {
+                PyErr_NoMemory();
+                return NULL;
+            }
+
+            processIdInfo.ImageName.MaximumLength = (USHORT)bufferSize;
+            processIdInfo.ImageName.Buffer = buffer;
+
+            status = NtQuerySystemInformation(
+                SystemProcessIdInformation,
+                &processIdInfo,
+                sizeof(SYSTEM_PROCESS_ID_INFORMATION),
+                NULL);
+        } while ((status == STATUS_INFO_LENGTH_MISMATCH) &&
+                    (bufferSize <= maxBufferSize));
+    }
+    else if (status == STATUS_INFO_LENGTH_MISMATCH) {
         // Required length is stored in MaximumLength.
         FREE(buffer);
         buffer = MALLOC_ZERO(processIdInfo.ImageName.MaximumLength);
-        if (! buffer)
-            return PyErr_NoMemory();
+        if (! buffer) {
+            PyErr_NoMemory();
+            return NULL;
+        }
+
         processIdInfo.ImageName.Buffer = buffer;
 
         status = NtQuerySystemInformation(
@@ -1281,6 +1284,7 @@ psutil_users(PyObject *self, PyObject *args) {
                 goto error;
         }
         else {
+            Py_INCREF(Py_None);
             py_address = Py_None;
         }
 
@@ -1611,8 +1615,8 @@ PsutilMethods[] = {
      "Determine if the process exists in the current process list."},
     {"cpu_count_logical", psutil_cpu_count_logical, METH_VARARGS,
      "Returns the number of logical CPUs on the system"},
-    {"cpu_count_phys", psutil_cpu_count_phys, METH_VARARGS,
-     "Returns the number of physical CPUs on the system"},
+    {"cpu_count_cores", psutil_cpu_count_cores, METH_VARARGS,
+     "Returns the number of CPU cores on the system"},
     {"boot_time", psutil_boot_time, METH_VARARGS,
      "Return the system boot time expressed in seconds since the epoch."},
     {"virtual_mem", psutil_virtual_mem, METH_VARARGS,
@@ -1666,12 +1670,12 @@ PsutilMethods[] = {
      "Stop a service"},
 
     // --- windows API bindings
-    {"win32_QueryDosDevice", psutil_win32_QueryDosDevice, METH_VARARGS,
+    {"QueryDosDevice", psutil_QueryDosDevice, METH_VARARGS,
      "QueryDosDevice binding"},
 
     // --- others
-    {"set_testing", psutil_set_testing, METH_NOARGS,
-     "Set psutil in testing mode"},
+    {"set_debug", psutil_set_debug, METH_VARARGS,
+     "Enable or disable PSUTIL_DEBUG messages"},
 
     {NULL, NULL, 0, NULL}
 };
@@ -1732,8 +1736,6 @@ void init_psutil_windows(void)
 
     if (psutil_setup() != 0)
         INITERROR;
-    if (psutil_load_globals() != 0)
-        INITERROR;
     if (psutil_set_se_debug() != 0)
         INITERROR;
 
