@@ -14,22 +14,22 @@ import time
 from collections import namedtuple
 
 from . import _common
+from ._common import ENCODING
+from ._common import ENCODING_ERRS
 from ._common import AccessDenied
+from ._common import NoSuchProcess
+from ._common import TimeoutExpired
 from ._common import conn_tmap
 from ._common import conn_to_ntuple
 from ._common import debug
-from ._common import ENCODING
-from ._common import ENCODING_ERRS
 from ._common import isfile_strict
 from ._common import memoize
 from ._common import memoize_when_activated
-from ._common import NoSuchProcess
 from ._common import parse_environ_block
-from ._common import TimeoutExpired
 from ._common import usage_percent
+from ._compat import PY3
 from ._compat import long
 from ._compat import lru_cache
-from ._compat import PY3
 from ._compat import range
 from ._compat import unicode
 from ._psutil_windows import ABOVE_NORMAL_PRIORITY_CLASS
@@ -39,6 +39,7 @@ from ._psutil_windows import IDLE_PRIORITY_CLASS
 from ._psutil_windows import NORMAL_PRIORITY_CLASS
 from ._psutil_windows import REALTIME_PRIORITY_CLASS
 
+
 try:
     from . import _psutil_windows as cext
 except ImportError as err:
@@ -197,7 +198,7 @@ def convert_dos_path(s):
         "C:\Windows\systemew\file.txt"
     """
     rawdrive = '\\'.join(s.split('\\')[:3])
-    driveletter = cext.win32_QueryDosDevice(rawdrive)
+    driveletter = cext.QueryDosDevice(rawdrive)
     remainder = s[len(rawdrive):]
     return os.path.join(driveletter, remainder)
 
@@ -241,8 +242,16 @@ def virtual_memory():
 def swap_memory():
     """Swap system memory as a (total, used, free, sin, sout) tuple."""
     mem = cext.virtual_mem()
-    total = mem[2]
-    free = mem[3]
+
+    total_phys = mem[0]
+    free_phys = mem[1]
+    total_system = mem[2]
+    free_system = mem[3]
+
+    # Despite the name PageFile refers to total system memory here
+    # thus physical memory values need to be substracted to get swap values
+    total = total_system - total_phys
+    free = min(total, free_system - free_phys)
     used = total - free
     percent = usage_percent(used, total, round_=1)
     return _common.sswap(total, used, free, percent, 0, 0)
@@ -304,9 +313,9 @@ def cpu_count_logical():
     return cext.cpu_count_logical()
 
 
-def cpu_count_physical():
-    """Return the number of physical CPU cores in the system."""
-    return cext.cpu_count_phys()
+def cpu_count_cores():
+    """Return the number of CPU cores in the system."""
+    return cext.cpu_count_cores()
 
 
 def cpu_stats():
@@ -759,7 +768,7 @@ class Process(object):
                 # 24 = ERROR_TOO_MANY_OPEN_FILES. Not sure why this happens
                 # (perhaps PyPy's JIT delaying garbage collection of files?).
                 if err.errno == 24:
-                    debug("%r forced into AccessDenied" % err)
+                    debug("%r translated into AccessDenied" % err)
                     raise AccessDenied(self.pid, self._name)
                 raise
         else:
