@@ -20,24 +20,25 @@ from psutil import BSD
 from psutil import FREEBSD
 from psutil import NETBSD
 from psutil import OPENBSD
-from psutil.tests import spawn_testproc
 from psutil.tests import HAS_BATTERY
+from psutil.tests import TOLERANCE_SYS_MEM
 from psutil.tests import PsutilTestCase
 from psutil.tests import retry_on_failure
 from psutil.tests import sh
-from psutil.tests import TOLERANCE_SYS_MEM
+from psutil.tests import spawn_testproc
 from psutil.tests import terminate
 from psutil.tests import unittest
 from psutil.tests import which
 
 
 if BSD:
-    PAGESIZE = os.sysconf("SC_PAGE_SIZE")
-    if os.getuid() == 0:  # muse requires root privileges
-        MUSE_AVAILABLE = which('muse')
-    else:
-        MUSE_AVAILABLE = False
+    from psutil._psutil_posix import getpagesize
+
+    PAGESIZE = getpagesize()
+    # muse requires root privileges
+    MUSE_AVAILABLE = True if os.getuid() == 0 and which('muse') else False
 else:
+    PAGESIZE = None
     MUSE_AVAILABLE = False
 
 
@@ -474,6 +475,7 @@ class FreeBSDSystemTestCase(PsutilTestCase):
                 psutil.sensors_temperatures()["coretemp"][cpu].high,
                 sysctl_result)
 
+
 # =====================================================================
 # --- OpenBSD
 # =====================================================================
